<!--
  Copyright (C) 2010 Orbeon, Inc.

  This program is free software; you can redistribute it and/or modify it under the terms of the
  GNU Lesser General Public License as published by the Free Software Foundation; either version
  2.1 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU Lesser General Public License for more details.

  The full text of the license is available at http://www.gnu.org/copyleft/lesser.html
  -->
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:f="http://orbeon.org/oxf/xml/formatting"
                xmlns:xh="http://www.w3.org/1999/xhtml"
                xmlns:xf="http://www.w3.org/2002/xforms"
                xmlns:xxf="http://orbeon.org/oxf/xml/xforms"
                xmlns:version="java:org.orbeon.oxf.common.Version"
                xmlns:xi="http://www.w3.org/2001/XInclude"
                xmlns:xxi="http://orbeon.org/oxf/xml/xinclude"
                xmlns:p="http://www.orbeon.com/oxf/pipeline">

    <xsl:import href="theme-plain.xsl"/>

    <!-- Try to obtain a meaningful title for the example -->
    <xsl:variable name="title" select="if (/xh:html/xh:head/xh:title != '')
                                       then /xh:html/xh:head/xh:title
                                       else if (/xh:html/xh:body/xh:h1)
                                            then (/xh:html/xh:body/xh:h1)[1]
                                            else '[Untitled]'" as="xs:string"/>

    <xsl:template match="xh:head">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
			<xh:link rel="stylesheet" href="/forms/assets/styles/mpn-overrides.css" type="text/css"/>
            <xh:link rel="stylesheet" href="/fr/style/bootstrap/css/bootstrap.css" type="text/css"/>
            <xh:link rel="stylesheet" href="/fr/style/form-runner-bootstrap-override.css" type="text/css"/>
            <xh:link rel="stylesheet" href="/config/theme/examples.css" type="text/css" media="all"/>
            <!-- PNCSSF-1037 -->
            <xh:script src="/ops/jquery/jquery-1.11.0.js"></xh:script>
			<xh:script src="/ops/jquery/jquery-migrate-1.2.1.js"></xh:script>
			<xh:script src="/apps/fr/style/bootstrap/js/bootstrap.js"></xh:script>
			<!-- END PNCSSF-1037 -->
            <xh:link rel="shortcut icon" href="/forms/assets/images/favicon16.ico"/>
            <xh:link rel="icon" href="/forms/assets/images/favicon16.png" type="image/png"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="xh:body">
        <xsl:copy>
            <xsl:apply-templates select="@* except @class"/>
            <xsl:attribute name="class" select="string-join(('orbeon', @class), ' ')"/>
            <xh:div class="container">
                <xh:div class="navbar navbar-inverse">
                    <xh:div class="navbar-inner">
                        <xh:div class="container">
                        	<!--PNCSSF-812 and PNCSSF-815 Get the "mpn-displayname" and "mpn-station" attribute" -->
								<xsl:variable name="headers" select="doc('input:request')/request/headers/header"/>
								<xsl:variable name="displayName" select="$headers[name='mpn-displayname']/value/string()"/>
								<xsl:variable name="stationName" select="$headers[name='mpn-station']/value/string()"/>
							<!-- END PNCSSF-812 and PNCSSF-815  -->

                            <xh:img src="/forms/assets/images/VAHealthLogoRGB.png" alt="VA Health"/>
                            <xh:img class="mpn_icon" alt="" src="/forms/assets/images/icon_mpn_v1.png"/>
                            <xh:h1><xsl:value-of select="$title"/></xh:h1>
	                        
							<!--PNCSSF-964 login/logout, user guide, and about links -->
							<xh:ul class="nav navlinks">
								<xsl:choose>
									<!-- logged in -->
									 <xsl:when test="$displayName != ''">
									 	<xh:li>
									 		<xh:a tabindex="0" class="navbar-links" href="#aboutModal" data-target="#aboutModal" data-toggle="modal" alt="Information about the Modernized Progress Notes Application" title="Information about the Modernized Progress Notes Application">About</xh:a>
								 		</xh:li>
										<xh:li>
											<xh:a tabindex="0" class="navbar-links" href="#helpModal" data-target="#helpModal" data-toggle="modal" alt="Modernized Progress Notes Help" title="Modernized Progress Notes Help">Help</xh:a>
										</xh:li>
										<xh:li>
											<xh:a tabindex="0" class="navbar-links" href="/halogout" alt="Log out of Modernized Progress Notes" title="Log out of Modernized Progress Notes">Logout</xh:a>
										</xh:li>
									 </xsl:when>
									 <!-- not logged in -->
									 <xsl:otherwise>
									 	<xh:li>
									 		<xh:a class="navbar-links" href="/login" alt="Log in to the MPN Application using your VistA credentials" title="Log in to the MPN Application using your VistA credentials">Login</xh:a>
									 	</xh:li>
									 </xsl:otherwise>
								</xsl:choose>
       						</xh:ul>
       						<!-- PNCSSF-1037 -->
       						<xh:div id="aboutModal" class="modal hide fade" tabindex="-1" role="dialog" aria-labelledby="aboutModalLabel" aria-hidden="true">
								<xh:div class="modal-body">
									<xh:span class="screen-reader-only">Beginning of Dialog</xh:span>
									<xh:h4 id="aboutModalLabel">About</xh:h4>
									<xh:table id="aboutTable">
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">Application Name</xh:th>
											<xh:td>Modernized Progress Notes</xh:td>
										 </xh:tr>
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">Version</xh:th>
											<xh:td>1.0</xh:td>
										 </xh:tr>
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">Developed By</xh:th>
											<xh:td>U.S. Department of Veterans Affairs</xh:td>
										 </xh:tr>
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">National Release Date</xh:th>
											<xh:td>06/01/2015</xh:td>
										 </xh:tr>
										 <xh:tr>
											<xh:th scope="row">Application Description</xh:th>
											<xh:td>VA Modernized Progress Notes (MPN) Form Builder (formerly known as Progress Note Construction Set) allows end users to build and deploy medical forms quickly and efficiently from desktop web browsers. VA Modernized Progress Notes (MPN) Form Runner allows clinicians to complete and update medical forms created in VA MPN Form Builder to capture patient data. VA MPN Form Builder is designed to be used on a desktop while VA MPN Form Runner can be used on either a desktop or tablet.  Both applications are operating system agnostic.</xh:td>
										 </xh:tr>
									</xh:table>
									<xh:button 
										class="btn btn-primary autowidth-button" 
										data-dismiss="modal" 
										alt="Close the About window" 
										title="Close the About Window" 
										onblur="$('#aboutModal').focus();">Close</xh:button>
									<xh:span class="screen-reader-only">End of Dialog</xh:span>
								</xh:div>
							</xh:div>
							<!--END PNCSSF-964 -->
							<!-- PNCSSF-1029 -->
							<xh:div id="helpModal" class="modal hide fade" tabindex="-1" role="dialog" aria-labelledby="helpModalLabel" aria-hidden="true">
								<xh:div class="modal-body">
									<xh:span class="screen-reader-only">Beginning of Dialog</xh:span>
									<xh:h4 id="helpModalLabel">Help</xh:h4>
									<xh:table id="helpTable">
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">Browsers</xh:th>
											<xh:td>
												This application can be used on the following browsers:
												<xh:ul>
													<xh:li>Internet Explorer 10 and higher</xh:li>
													<xh:li>Safari 7 and higher</xh:li>
													<xh:li>Firefox 24 and higher</xh:li>
													<xh:li>Chrome 30 and higher</xh:li>
												</xh:ul>
											</xh:td>
										 </xh:tr>
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">User Manual</xh:th>
											<xh:td>
												<xh:div class="float-left"><xh:a target="_blank" href="forms/assets/docs/VA_MPN_Form_Builder_UM.pdf" alt="Download the User Manual for VA MPN Form Builder" title="Download the User Manual for VA MPN Form Builder">VA MPN Form Builder</xh:a></xh:div>
												<xh:div class="float-right" style="padding-right:15px"><xh:a target="_blank" href="forms/assets/docs/VA_MPN_Form_Runner_UM.pdf" alt="Download the User Manual for VA MPN Form Runner" title="Download the User Manual for VA MPN Form Runner">VA MPN Form Runner</xh:a></xh:div>
											</xh:td>
										 </xh:tr>
										 <xh:tr class="bottom-border-tr">
											<xh:th scope="row">Toll Free Help Desk Weekdays 7 a.m.-7 p.m. (CT)</xh:th>
											<xh:td>For help with this app contact the VA Mobile App Help Desk at TBD.</xh:td>
										 </xh:tr>
										 <xh:tr>
											<xh:th scope="row">General Feedback to VA</xh:th>
											<xh:td>General Feedback to VA link TBD</xh:td>
										 </xh:tr>
									</xh:table>
									<xh:button 
										class="btn btn-primary autowidth-button" 
										data-dismiss="modal" 
										alt="Close the Help window" 
										title="Close the Help Window"
										onblur="$('#helpModal').focus();">Close</xh:button>
									<xh:span class="screen-reader-only">End of Dialog</xh:span>
								</xh:div>
							</xh:div>
       						<!-- END PNCSSF-1029  -->
							<!-- PNCSSF-1123 -->
							<xh:div id="sessionModal" class="modal hide fade" tabindex="-1" role="dialog" aria-labelledby="sessionModalLabel" aria-hidden="true">
								<xh:div class="modal-body">
									<xh:span class="screen-reader-only">Beginning of Dialog</xh:span>
									<xh:h4 id="sessionModalLabel">Session Timeout Warning</xh:h4>
									<xh:p>
										Your session will end in 2 minutes. Press OK to stay connected to this application.
									</xh:p>
									<xh:button 
										class="btn btn-primary autowidth-button" 
										data-dismiss="modal" 
										alt="OK" 
										title="OK"
										onblur="$('#sessionModal').focus();">OK</xh:button>
									<xh:span class="screen-reader-only">End of Dialog</xh:span>
								</xh:div>
							</xh:div>
							<!-- END PNCSSF-1123 -->
							
							<!--PNCSSF-812 and PNCSSF-815 Get the "mpn-displayname" and "mpn-station" attribute and use it to replace instance "displayname-instance" -->
							<xsl:if test="$displayName != ''">
								<xh:div id="welcomeMessage">
									Welcome <xh:span class="boldSpan"><xsl:value-of select="$displayName"/></xh:span>
									<xsl:if test="$stationName != ''">
										<xh:div id="stationDiv">
											Station <xh:span class="boldSpan"><xsl:value-of select="$stationName"/></xh:span>
										</xh:div>
									</xsl:if>
								</xh:div>
							</xsl:if>
	                        <!-- END PNCSSF-812 and PNCSSF-815  -->
                        </xh:div>
                    </xh:div>
                </xh:div>
                <xh:div id="body-container">
					<xsl:apply-templates select="node()"/>
					<xh:div id="push"></xh:div>
					 <xh:footer class="mpn-footer">
					  <xh:div>U.S. Department of Veterans Affairs | 810 Vermont Avenue, NW Washington DC 20420</xh:div>
					  <xh:div>Modernized Progress Notes (MPN)</xh:div>
					  <xh:div>Version 1.0</xh:div>
					</xh:footer>
				</xh:div>
            </xh:div>
            <xsl:if test="p:property('oxf.epilogue.show-feedback')">
                <xsl:copy-of select="doc('oxf:/config/feedback.xhtml')"/>
            </xsl:if>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
